@@ -32,6 +32,10 @@ class FirestoreClient:
         self.application = os.environ.get('APP_FIRESTORE_NAME',None)
         if not self.company or not self.application:
             raise Exception("The company name and application name must be set in environment variables ('APP_FIRESTORE_COMPANY','APP_FIRESTORE_NAME'")
+        user_info = self._get_users()
+        self.admins = user_info['admins']
+        self.sys_admins = user_info['sys_admins']
+        self.interactors = user_info['interactors']
 
     def _create_q_paths(self, project_name):
         project = project_name
@@ -40,6 +44,52 @@ class FirestoreClient:
 
     def _create_pubsub_topics(self, project_name):
         self.pubsub_topics = {x:f'projects/{project_name}/topics/{x}' for x in FirestoreClient.TOPICS}
+        
+    @property
+    def user_admin(self):
+        return self.user_email in self.admins
+    
+    @property
+    def user_sys_admin(self):
+        return self.user_email in self.sys_admins
+    
+    @property
+    def user_interactor(self):
+        return self.user_email in self.interactors
+    
+    def getAdmins(self):
+        return self.admins
+    
+    def getSysAdmins(self):
+        return self.sys_admins
+    
+    def getRole(self):
+        if self.user_sys_admin:
+            return "SysAdmin"
+        if self.user_admin:
+            return "Admin"
+        return "Interactor"
+    
+    def getRoleOrder(self):
+        return {'SysAdmin':3,'Admin':2,'Interactor':1}[self.getRole()]
+    
+    def _get_users(self):
+        usersCollection = self.fsClient.collection(f'application_data/{self.company}/Users')
+        snaps = [x.get() for x in usersCollection.list_documents()]
+        admins = []
+        sys_admins = []
+        interactors = []
+        for snap in snaps:
+            isAdmin = snap.get('admin')
+            isSysAdmin = snap.get('sys_admin')
+            users = snap.get('users')
+            for user in users:
+                if isAdmin:
+                    admins.append(user)
+                if isSysAdmin:
+                    sys_admins.append(user)
+                interactors.append(user)
+        return {'admins':admins,'sys_admins':sys_admins,'interactors':interactors}
 
     @classmethod
     def getInstance(cls,inEmail=None,project_name=None):
