@@ -1,3 +1,28 @@
+###############################################################################################################################
+# Licensed to the part of the ownership of Analytics Supply LLC.
+#  All updates to this file should only be done at the sole discretion of the 
+#  officers of Analytics Supply:
+#  
+##################################
+##  Module Name:  grow_week.py
+##################################
+#
+#  Description:
+#  --  Plants are tracked by week and availability, sales and reserves need to be tracked by week as well.  \
+#      This module creates data entries by week that these things can be stored
+#
+##################################
+#
+#  Created:  ???  Somewhere around July 2020
+#
+##################################
+#  UPDATES:
+#  Date, Issue #, Name of Developer, Short description of bug
+#  9/15/2020, ct:85, Jason Bowles, Update to stop tracking items by type and name:  https://gitlab.com/AnalyticsSupply/customer-tracking/-/issues/85
+#
+#
+#
+#################################################################################################################################
 from .sales_inv_utils import SalesInvBase
 from datetime import datetime
 from datetime import timedelta
@@ -6,14 +31,14 @@ import jmespath
 from .item_week import ItemWeekCollection, ItemWeek
 from .item_reserve import ItemReserve
 from .reserve_summary import ReserveSummary
-from .inventory_active_items import InventoryActiveItems, InventoryItems
+from .inventory_active_items import InventoryActiveItems, InventoryItems, InventoryLocation
 
 '''
 This will be the new converted class GrowWeek
 '''
 class GrowWeek(SalesInvBase):
 
-    ext_fields = ['week_number','year','week_monday','plantgrow','vaseinv','soft_delete','parent_path','path']
+    ext_fields = ['week_number','year','week_monday','soft_delete','parent_path','path']
     COLLECTION_NAME = 'application_data'
     
     """ Represents a Week where we can have reserve orders """
@@ -31,6 +56,7 @@ class GrowWeek(SalesInvBase):
         self._itemweek = {}
         self._reserves = []
         self._reserve_summaries = []
+        self._reserve_summaryObj = None
 
         super(GrowWeek, self).__init__(fsClient, **kwargs)
     
@@ -52,18 +78,43 @@ class GrowWeek(SalesInvBase):
         docDict['fs_docSnap'] = snap
         docDict['fs_docRef'] = ref
         return GrowWeek(GrowWeek.get_firestore_client(),**docDict)
+
+    def _itemWeek_Lookup(self,invLoc, itemType):
+        '''
+        Unique id of the item week given its location and item type
+
+        Parameters
+        ----------
+            invLoc : str
+                The inventory location
+            itemType :  str
+                The item type for the inventory
+        '''
+        return f"{invLoc}__{itemType}"
     
-    def get_itemweek_type(self,item_type):
-        iw = self._itemweek.get(item_type,None)
+    def get_itemweek_type(self,invLoc, item_type):
+        '''
+        Get an itemweek given the inventory location and item type
+
+        Parameters
+        ----------
+            invLoc : str
+                The inventory location for the item week
+            item_type : str
+                The item type for the inventory
+        '''
+        itemTypeKey = self._itemWeek_Lookup(invLoc,item_type)
+        iw = self._itemweek.get(itemTypeKey,None)
         if iw is None:
-            item_path = self.path+'/Items/'+item_type
-            iw = ItemWeekCollection.getInstance(self._fsClient.document(item_path))
+            invPath = InventoryLocation.getInstance().getGrowWeekPath(self.path,invLoc)
+            item_path = f'{invPath}/Items/{item_type}'
+            iw = ItemWeekCollection.getInstance(self._fsClient.document(item_path),self)
             if not iw.exists:
                 iw.item_type = item_type
                 iw.finish_week = self.id
-                self._in_growWeekParent = self
+                iw._in_growWeekParent = self
                 iw.update_ndb(True)
-                self._itemweek[item_type] = iw
+                self._itemweek[itemTypeKey] = iw
         return iw
     
     @classmethod
@@ -107,12 +158,26 @@ class GrowWeek(SalesInvBase):
 
     @property
     def next_week(self):
-        nxtWk = datetime.fromisoformat(self.week_monday) + timedelta(days=7)
+        nxtWk = None
+        if isinstance(self.week_monday,str):
+            nxtWk = datetime.fromisoformat(self.week_monday) + timedelta(days=7)
+        elif isinstance(self.week_monday,datetime):
+            nxtWk = self.week_monday + timedelta(days=7)
+        else:
+            raise Exception("Date format of the Monday Date is Unknown")
+        
         return GrowWeek.GetGrowWeekByDate(nxtWk)
 
     @property
     def prior_week(self):
-        nxtWk = datetime.fromisoformat(self.week_monday) - timedelta(days=7)
+        nxtWk = None
+        if isinstance(self.week_monday,str):
+            nxtWk = datetime.fromisoformat(self.week_monday) - timedelta(days=7)
+        elif isinstance(self.week_monday,datetime):
+            nxtWk = self.week_monday - timedelta(days=7)
+        else:
+            raise Exception("Date format of the Monday Date is Unknown")
+
         return GrowWeek.GetGrowWeekByDate(nxtWk)
 
     @property
@@ -167,82 +232,34 @@ class GrowWeek(SalesInvBase):
             self._reserves = ItemReserve.get_reserves_for_week(self)
         return self._reserves
 
-    def getSummary(self):
+    def getSummaryOld(self):
         summary = self._documentRef.collection("ReservesSummary").document('summary')
         return ReserveSummary.getInstance(summary)
+    
+    def getSummary(self,item_type=None):
+        docId = item_type.replace(" ","_") if item_type is not None else 'summary'
+        summary = self._documentRef.collection("ReservesSummary").document(docId)
+        return ReserveSummary.getInstance(summary)
+    
+    def get_reserve_reference(self):
+        return self._documentRef.collection('Reserves').document(self._get_doc_id('Reserve')) 
 
-    def _get_item_info(self,item_type,item):
-        info = getattr(item,item_type)
-        ret = None
-        if isinstance(info,list):
-            # it's an array... so we can make some assumptions
-            ret = []
-            sing = (item_type)
-            for i in info:
-                qty = 1
-                itemEntry = None
-                if isinstance(i,dict):
-                    itemEntry = self._get_recipe_costing_item(item_type,i[sing])
-                    qty = i.get('qty',1)
-                else:
-                    itemEntry = self._get_recipe_costing_item(item_type,i)
-                ret.append({sing: itemEntry,'qty':qty})
-        else:
-            ret = self._get_recipe_costing_item(info)
-        return ret
-
-    def _get_reserve_info(self, item_id):
-        itemObj = self._get_sb_by_dnl(item_id)
-        item = self._get_item_info(itemObj)
-        location = self._get_loc_info(self._get_sb_instance_by_path(itemObj.parent_doc_path))
-        itemInfo = []
-        for itemType in self._inventoryItems.tracked_items:
-            entry = {}
-            entry['name'] = itemType
-            entry['info'] = self._get_item_info(itemType,item)
-            itemInfo.append(entry)
-
-        return location, item, itemInfo
-
-    def create_reserve(self,customer_id, location_id, item_id, reserved, reserveDate=None):
-        customer = self._get_cust_info(self._get_sb_by_dnl(customer_id))
-        location, item, itemInfo = self._get_reserve_info(item_id)
-        docRef = self._documentRef.collection('Reserves').document(self._get_doc_id('Reserve')) 
-        ir = ItemReserve.getInstance(docRef)
-        ir.customer = customer
-        ir.item = item
-        ir.location = location
-        ir.num_reserved = int(reserved)
-        ir.finish_week = self.id
-        ir._reserve_date = reserveDate
-        for i in itemInfo:
-            setattr(ir,i['name'],i['info'])
-            ir._add_field(i['name'])
-
-        ir.update_ndb(True)
-        self._insert_dnl(docRef)
-        return ir
+    def create_reserve(self,customer_id, location_id, item_id, reserved, reserveDate=None, inventory_location=None):
+        docRef = self.get_reserve_reference()
+        return ItemReserve.create_reserve(docRef,customer_id,location_id,item_id,reserved,self.id,reserveDate, inventory_location)
 
     def update_reserve(self, reserve_id, customer_id, location_id, item_id, reserved, reserveDate=None):
+        if reserveDate is not None:
+            gw = GrowWeek.GetGrowWeekByDate(reserveDate)
+        else:
+            gw = self
         ir = ItemReserve.getItemReserveInstance(reserve_id)
-        gw = GrowWeek.GetGrowWeekByDate(reserveDate)
-        if gw.id != ir.finish_week:
-            ir.delete_resp()
-            return self.create_reserve(customer_id, location_id, item_id, reserved, reserveDate)
+        return ir.update_reserve(customer_id,location_id, item_id, reserved, gw, reserveDate)
 
-        location, item, itemInfo = self._get_reserve_info(item_id)
-        customer = self._get_cust_info(self._get_sb_by_dnl(customer_id))
-        ir.num_reserved = int(reserved)
-        ir._reserve_date = reserveDate
-        ir.customer = customer
-        ir.location = location
-        for i in itemInfo:
-            setattr(ir,i['name'],i['info'])
-            ir._add_field(i['name'])
+    def update_item_info(self, reserve_id, item_id):
+        ir = ItemReserve.getItemReserveInstance(reserve_id)
+        return ir.refresh(refreshInvLoc=True)
 
-        ir.item = item
-        ir.update_ndb()
-        return ir
 
     @property
     def reserve_summaries(self):
@@ -263,13 +280,18 @@ class GrowWeek(SalesInvBase):
         '''
 
         if len(self._reserve_summaries) == 0:
-            summaryPath = self.path+'/ReservesSummary/summary'
-            docRef = self._fsClient.document(summaryPath)
-            snap = docRef.get()
-            if snap.exists:
-                self._reserve_summaries = snap.to_dict()['summary']   
+            if self._reserve_summaryObj is None:
+                self._reserve_summaryObj = ReserveSummary.getReserveSummary(self.id)
+            if self._reserve_summaryObj.exists:
+                self._reserve_summaries = self._reserve_summaryObj.summary  
 
         return self._reserve_summaries
+    
+    @property
+    def reserve_summaryObj(self):
+        if self._reserve_summaryObj is None:
+            self._reserve_summaryObj = ReserveSummary.getReserveSummary(self.id)
+        return self._reserve_summaryObj
 
     @classmethod
     def create_weeks(cls, start_date, end_date):
@@ -284,6 +306,10 @@ class GrowWeek(SalesInvBase):
     def create_doc_id(cls, year, wk_num):
         return str(year)+"_"+str(wk_num).zfill(2)
 
+    @classmethod
+    def GetGrowWeekNow(cls):
+        return cls.GetGrowWeekByDate(datetime.now())
+
     @classmethod
     def GetGrowWeekByDate(cls, indate):
         '''Get the week... if it doesn't exist..create it'''
@@ -305,14 +331,21 @@ class GrowWeek(SalesInvBase):
 
     @classmethod
     def _get_active_items(cls,item_type):
-        return InventoryActiveItems.get_all(item_type)
+        return InventoryActiveItems.get_all_active(item_type)
 
     @classmethod
-    def get_9_itemweek(cls, item_type, center_date):
+    def get_9_itemweek(cls,invLoc, item_type, center_date):
         """
         Get 10 weeks total 9 around the date provided
-        :param center_date:
-        :return: array of ItemWeek
+
+        Parameters
+        ----------
+            invLoc : str
+                The location of the inventory
+            item_type : str
+                The item type for the inventory
+            center_date : date
+                The date from which to go 4 weeks in either direction
         """
         activeItems = GrowWeek._get_active_items(item_type)
         gws = GrowWeek.get_9_weeks(center_date)
@@ -321,41 +354,92 @@ class GrowWeek(SalesInvBase):
             gwd = gw.get_growweek_dict()
             gwd['inv_items'] = []
             for item in activeItems.keys():
-                gwd['inv_items'].append(gw.get_or_create_itemweek(item_type,activeItems[item]).get_itemweek_dict())
+                gwd['inv_items'].append(gw.get_or_create_itemweek(invLoc,item_type,activeItems[item]).get_itemweek_dict())
             resp['data'].append(gwd)
 
         return resp
 
-    def _getItemWeekCollection(self, item_type):
-        itemWeekCollection = self._itemweek.get(item_type,None)
+    def _getItemWeekCollection(self, invLoc : str, item_type : str) -> ItemWeekCollection:
+        '''
+        <private>
+        Get the item week collection
+
+        Parameters
+        ----------
+            invLoc : str
+                the inventory location for the item week collection
+            item_type : str
+                The item type for the itemWeekCollection
+        '''
+        itemWeekKey = self._itemWeek_Lookup(invLoc,item_type)
+        itemWeekCollection = self._itemweek.get(itemWeekKey,None)
         if itemWeekCollection is None:
-            itemWeekCollection = ItemWeekCollection.getOrCreateInstance(item_type,self)
-            self._itemweek[item_type] = itemWeekCollection
+            itemWeekCollection = ItemWeekCollection.getOrCreateInstance(invLoc, item_type,self)
+            self._itemweek[itemWeekKey] = itemWeekCollection
         return itemWeekCollection
     
-    def get_or_create_itemweek(self,item_type,itemObj):
-        itemWeekCollection = self._getItemWeekCollection(item_type)
+    def get_or_create_itemweek(self,invLoc,item_type,itemObj):
+        itemWeekCollection = self._getItemWeekCollection(invLoc, item_type)
         return itemWeekCollection.create_itemweek_entry(itemObj,self)
 
     def chk_create_item(self, din, name, item, item_type):
         if name not in din.keys():
             din[name] = {'wanted':0,'actual':0,'forecast':0, 'reserved':0, 'notes':0}
             din[name]['item_key'] = item.id
+            din[name]['item_name'] = item.name
             din[name]['week_key'] = self.id
 
     @classmethod
-    def update_itemweek_supply(cls, item_type, weekId, supply_id, supplier, forecast, confirmation_num, cost):
+    def update_itemweek_supply(cls, invLoc, item_type, weekId, supply_id, supplier, forecast, confirmation_num, cost):
+        '''
+        Update the supply for an instance of item week
+
+        Parameters
+        ----------
+            invLoc : str
+                The location of the inventory
+            item_type : str
+                The type of item for the inventory
+            weekId : str
+                The grow week id for this supply <year>_<week number>
+            supply_id : str
+                The unique id for the supply that is being updated
+            supplier : str
+                The supplier name
+            forecast : int
+                The amount that is forecasted to be available for this week_id
+            confirmation_num : str
+                The confirmation code/num that identifies this supply
+            cost : float
+                The cost of this supply of inventory
+        '''
         gw = GrowWeek.getGrowWeekInstance(weekId)
-        itemWeekCollection = gw._getItemWeekCollection(item_type)
+        itemWeekCollection = gw._getItemWeekCollection(invLoc, item_type)
         supply = itemWeekCollection.get_supply().getSupplyById(supply_id)
         return supply.update(supplier, forecast, confirmation_num, cost)
 
     @classmethod
-    def update_itemweek(cls, itemName, item_type, weekId, actual):
+    def update_itemweek(cls, invLoc, itemName, item_type, weekId, actual):
+        '''
+        Update the amount of inventory for this item and this week
+
+        Parameters
+        ----------
+            invLoc : str
+                The inventory location
+            itemName : str
+                The name of the item being tracked
+            item_type : str
+                The item type of the inventory
+            weekId : str
+                The week id for the item week <year>_<week number>
+            actual : int
+                The actual inventory amount
+        '''
         resp = {'status':'success','msg':'PlantGrow Updated Successfully'}
         try:
             gwRef = GrowWeek.getGrowWeekInstance(weekId)
-            itemWeekCollection = gwRef._getItemWeekCollection(item_type)
+            itemWeekCollection = gwRef._getItemWeekCollection(invLoc, item_type)
             iw = itemWeekCollection.get_itemweek(itemName)
             if iw.exists:
                 #pg.want_qty = int(wanted)
@@ -375,12 +459,22 @@ class GrowWeek(SalesInvBase):
             resp = {'status':'failed','msg': str(e)}
         return resp
 
-    def week_summary(self,item_type):
+    def week_summary(self,invLoc, item_type):
+        '''
+        Get the summary for this item_type in this location
+
+        Parameters
+        ----------
+            invLoc : str
+                The inventory location for the inventory
+            item_type : str
+                The item type for this inventory
+        '''
         iws = []
         items = list(self._get_active_items(item_type).values())
+
         for item in items:
-            if item.name != 'N/A':
-                iws.append({'key': item.name, 'value': self.get_or_create_itemweek(item_type,item)})
+            iws.append({'key': item.id, 'value': self.get_or_create_itemweek(invLoc, item_type,item)})
         
 
         iwd = {}
@@ -388,6 +482,7 @@ class GrowWeek(SalesInvBase):
             name = pg['key']
             itemweek = pg['value']
             self.chk_create_item(iwd,name,itemweek,item_type)
+            iwd[name]['inventory_location'] = invLoc
             iwd[name]['actual'] = iwd[name]['actual'] + itemweek.actual
             iwd[name]['notes'] = len(itemweek.notes)
             iwd[name]['id'] = itemweek.id
@@ -395,28 +490,7 @@ class GrowWeek(SalesInvBase):
             supps = itemweek.supply
             for supp in supps:
                 iwd[name]['forecast'] = iwd[name]['forecast'] + supp.forecast
-            '''
-            This is what should get returned... 
-            [
-                {
-                    "reserved": 360,
-                    "qty": 1
-                },
-                {
-                    "reserved": 800,
-                    "qty": 1
-                }
-            ]
-            '''
-            rsvs = []
-            item_key = ItemWeek.CleanItemType(item_type)+'s'
-            item_ent = itemweek.clean_item_type
-            #"[?vases[?vase == '3\" Designer']].{reserved: num_reserved, qty: vases[*].qty | [0]}"
-            srch = '[?'+item_key+'[?'+item_ent+' == \''+name+'\']].{reserved: num_reserved, qty: '+item_key+'[*].qty | [0]}'
-            rsvs = jmespath.search(srch, self.reserve_summaries)
-
-            for rsv in rsvs:
-                iwd[name]['reserved'] = iwd[name]['reserved'] + (rsv['qty'] * rsv['reserved'])
+            iwd[name]['reserved'] = self.reserve_summaryObj.getReserveAmtByItem(item_type,name,invLoc)
 
 
         for key in iwd.keys():
@@ -428,40 +502,60 @@ class GrowWeek(SalesInvBase):
     @classmethod
     def _split_item_id(cls,itemId):
         '''
-        Item id:  2020_18-Singolo
+        Split up the key to get needed info
+        Item id:  2020_18__Virginia__recipe_costing-157
         Item Type: Plants
         '''
-        parts = itemId.split("-")
-        week_id = parts[0]
-        item_name = None
-        if len(parts) > 1:
-            item_name = parts[1]
-        return item_name, week_id
+        keyParts = ItemWeek.ParseItemWeekId(itemId)
+        return keyParts['item_id'],keyParts['week_id'],keyParts['inventory_location']
 
     @classmethod
-    def get_itemweek_by_id(cls,item_type, itemId):
+    def get_itemweek_by_id(cls,itemType, itemId):
         '''  Assuming that the PlantGrow Id is Week<->Plant Name
         2020_20-Bonita
         '''
-        itemName, weekId = GrowWeek._split_item_id(itemId)
-        if itemName is not None and weekId is not None:
-            #return GrowWeek.get_plantgrow(plName,weekId)
-            return GrowWeek.get_itemweek(item_type, itemName, weekId)
+        itemId, weekId, invLoc = GrowWeek._split_item_id(itemId)
+        if itemId is not None and weekId is not None:
+            #return GrowWeek.get_itemweek(invLoc, itemId, weekId)
+            return GrowWeek.get_itemweek_typ(invLoc, itemType, itemId, weekId)
         raise Exception("Invalid plant grow id supplied: {}".format(itemId))
 
     @classmethod
-    def get_itemweek(cls, argItemType, argItem, argWeek):
+    def get_itemweek_by_idtyp(cls,itemType, itemId):
+        '''  Assuming that the PlantGrow Id is Week<->Plant Name
+        2020_20-Bonita
+        '''
+        itemId, weekId, invLoc = GrowWeek._split_item_id(itemId)
+        if itemId is not None and weekId is not None:
+            return GrowWeek.get_itemweek_typ(invLoc, itemType, itemId, weekId)
+        raise Exception("Invalid plant grow id supplied: {}".format(itemId))
+
+    @classmethod
+    def get_itemweek(cls, invLoc, argItemId, argWeek):
+        iwKey = ItemWeek.CreateItemWeekId(argWeek,invLoc,argItemId)
+        return cls.get_itemweek_by_id(iwKey)
+        #gw = GrowWeek.getGrowWeekInstance(argWeek)
+        #item = GrowWeek.GetRecipeItemById(argItemId)
+        #iwc = gw._getItemWeekCollection(invLoc, item['item_type'])
+        #iw = iwc.get_itemweek(item['id'])
+        #return iw
+
+    @classmethod
+    def get_itemweek_typ(cls, invLoc, argItemType, argItemId, argWeek):
         gw = GrowWeek.getGrowWeekInstance(argWeek)
-        iwc = gw._getItemWeekCollection(argItemType)
-        iw = iwc.get_itemweek(argItem)
+        iwc = gw._getItemWeekCollection(invLoc,argItemType)
+        iw = iwc.get_itemweek(argItemId)
+        if iw is None:
+            item = cls.returnFSDocByPath(f'application_data/Color_Orchids/Customer_Tracking/StorageBlob/recipe_costing/{argItemId}')
+            iw = iwc.create_itemweek_entry(item,gw)
         return iw
 
     @classmethod
-    def get_item_availability_now(cls,item_type, item_name):
+    def get_item_availability_now(cls,invLoc, item_type, item_name):
         cal = datetime.now()
 
         gw = GrowWeek.GetGrowWeekByDate(cal)
-        iwc = gw._getItemWeekCollection(item_type)
+        iwc = gw._getItemWeekCollection(invLoc, item_type)
         iw = iwc.get_itemweek(item_name)
         avail = {"prior":0,"current":0,"next":0}
         if iw:
@@ -479,3 +573,8 @@ class GrowWeek(SalesInvBase):
     def save_note(cls,note, item_type, item_key):
         iw = GrowWeek.get_itemweek_by_id(item_type,item_key)
         return iw.create_note(note)
+
+    @classmethod
+    def save_note_typ(cls,note, item_type, item_key):
+        iw = GrowWeek.get_itemweek_by_idtyp(item_type, item_key)
+        return iw.create_note(note)
