@@ -1,8 +1,10 @@
+import re
 from google.api_core.protobuf_helpers import get
+
 from .storage_blob import StorageBlob
 from .sales_inv_utils import SalesInvBase
 from .item_week import ItemWeek
-from .data_storage_type import StorageField
+from . import FSDocument
 from .inventory_active_items import InventoryTracking, InventoryLocation
 from datetime import datetime
 import logging
@@ -51,12 +53,15 @@ class ItemReserve(SalesInvBase):
     
     """
 
-    ext_fields = ['customer','location','inventory_location','inv_loc_override','finish_week', 'reserve_date','num_reserved','item','recipe_items','soft_delete','parent_path','path']
+    ext_fields = ['customer','location','inventory_location','inv_loc_override','finish_week', 'reserve_date',
+                  'num_reserved','item','recipe_items','custom_plant_item','soft_delete','parent_path','path']
+    
     COLLECTION_NAME = 'application_data'
     NAME = 'Reserves'
 
     def __init__(self, fsClient, **kwargs):
         self.soft_delete = kwargs.get('soft_delete',False)
+        self.custom_plant_item = kwargs.get('custom_plant_item',False)
         self.customer = kwargs.get('customer',None) 
         self.location = kwargs.get('location',None)
         self._location_id = None
@@ -93,6 +98,10 @@ class ItemReserve(SalesInvBase):
     @classmethod
     def __growWeekBasePath(cls):
         return ItemReserve.COLLECTION_NAME+'/'+ItemReserve.get_client().company+'/Sales_Inventory/Converted/GrowWeek'
+    
+    @classmethod
+    def _getReservePath(cls,finishWeek,reserveId):
+        return f"{cls.__growWeekBasePath()}/{finishWeek}/Reserves/{reserveId}"
 
     @property
     def _growWeekParent(self):
@@ -198,9 +207,10 @@ class ItemReserve(SalesInvBase):
         return ItemReserve(ItemReserve.get_firestore_client(),**docDict)
 
     @classmethod
-    def getItemReserveInstance(cls,reserveId):
-        pr = ItemReserve.GetByDNL(reserveId,ItemReserve)
-        return pr
+    def getItemReserveInstance(cls,reserveId, growWeek=None):
+        if growWeek is None:
+            return ItemReserve.GetByDNL(reserveId,ItemReserve)
+        return cls.getInstance(FSDocument.getInstance(cls._getReservePath(growWeek,reserveId)))
     
     @classmethod
     def getMatchingReservesByItem(cls,item_id, week_id=None):
@@ -267,24 +277,6 @@ class ItemReserve(SalesInvBase):
         docRefs = col.list_documents()
         return [ItemReserve.getInstance(x) for x in docRefs]
 
-    #def get_db_reserve2(self):
-    #    prdb = {}
-    #    prdb['_id'] = self.id
-    #    prdb['plant'] = self.plants[0]['name']
-    #    prdb['vase'] = self.vase['name']
-    #    prdb['product'] = self.item['name']
-    #    prdb['plant_id'] = self.plants[0]['id']
-    #    prdb['vase_id'] = self.vase['id']
-    #    prdb['product_id'] = self.item['id']
-    #    prdb['num_reserved'] = self.num_reserved
-    #    prdb['week_id'] = self.finish_week
-    #    prdb['customer'] = self.customer['name']
-    #    prdb['customer_id'] = self.customer['id']
-    #    prdb['sales_rep'] = self.added_by
-    #    prdb['add_date'] = self.timestamp
-    #    prdb['soft_delete'] = "Y" if self.soft_delete and (self.soft_delete == True) else "N"
-    #    return prdb
-
     @classmethod
     def hardDelete(cls,argId):
         pr = SalesInvBase.GetByDNL(argId,ItemReserve)
@@ -303,80 +295,26 @@ class ItemReserve(SalesInvBase):
             return resp
 
     @classmethod
-    def update(cls,argId, argCustId, argLocId, argWeekId, argItemId, argReserved=0):
-        """ pr = DataNumberLookup.get_obj_by_dnl(argId,ItemReserve)
+    def CreateCustomItem(cls,reserveId,itemId=None,growWeek=None):
+        ir = ItemReserve.getItemReserveInstance(reserveId,growWeek)
+        if ir is None:
+            return None
+        resp = {'reserve':ir,'item': ir._createCustomItem(itemId)}
+        return resp
+    
+    def _createCustomItem(self,itemId=None):
+        self.custom_plant_item = True
         
-        if pr:
-            if pr.finish_week != argWeek.id:
-                if ItemReserve.hardDelete(argId):
-                    return  ItemReserve.add(argCustomer,argLocation,argWeek,argItem,argReserved,argId)
-                else:
-                    raise Exception("Could not delete reserve: "+argId)
-
-            pr.customer = {}
-            pr.customer['name'] = argCustomer.customer_name
-            pr.customer['id'] = argCustomer.data_number_lookup
-            pr.customer['path'] = argCustomer.path
-            pr.customer['type'] = "Item_Tracking"
-
-            pr.location = {}
-            pr.location['name'] = argLocation.location_name
-            pr.location['id'] = argLocation.data_number_lookup
-            pr.location['path'] = argLocation.path
-            pr.location['type'] = "Item_Tracking"
-
-            pr.item = {}
-            pr.item['name'] = argItem.Product_Name
-            pr.item['id'] = argItem.data_number_lookup
-            pr.item['path'] = argItem.path
-            pr.item['type'] = "Item_Tracking"
-
-            pr.num_reserved = int(argReserved)
-        return pr.update_ndb() """
-        raise Exception("Item Reserve: update not implemented")
-
-    @classmethod
-    def add(cls,argCustomer, argLocation, argWeek, argItem, argReserved=0, argDnl=None):
-        '''
-        dnl = argDnl
-        if dnl is None:
-            dnl = SalesInvBase.GetNextDNL(ItemReserve.NAME)
-
-        data = {}
-        data['customer'] = {}
-        data['customer']['name'] = argCustomer.customer_name
-        data['customer']['id'] = argCustomer.data_number_lookup
-        data['customer']['path'] = argCustomer.path
-        data['customer']['type'] = "Item_Tracking"
-
-        data['location'] = {}
-        data['location']['name'] = argLocation.location_name
-        data['location']['id'] = argLocation.data_number_lookup
-        data['location']['path'] = argLocation.path
-        data['location']['type'] = "Item_Tracking"
-
-        data['finish_week'] = argWeek.id
-
-        data['item'] = {}
-        data['item']['name'] = argItem.Product_Name
-        data['item']['id'] = argItem.data_number_lookup
-        data['item']['path'] = argItem.path
-        data['item']['type'] = "Item_Tracking"
-
-        data['num_reserved'] = int(argReserved)
-
-        path = argWeek.path+'/'+ItemReserve.NAME+'/'+dnl
-        docRef = ItemReserve.get_firestore_client().document(path)
-
-        data['fs_docRef'] = docRef
-
-        pr = ItemReserve(ItemReserve.get_firestore_client(),**data)
-        resp = pr.update_ndb(True)
-
-        SalesInvBase.AddDNL(dnl, docRef.path)
+        if itemId is None:
+            itemId = self.item['id']
+            
+        newItemObj = StorageBlob.createItemCopy(itemId,dnlExt=self.dnl)
         
-        return resp'''
-        raise Exception("Item Reserve: add not implemented, Removed 10/26/2021")
+        if newItemObj is None:
+            raise Exception("Unable to copy an item that can't be found: "+itemId)
+        self.item = ItemReserve.GetItemInfo(newItemObj)
+        self.reference.update({"item":self.item, "custom_plant_item":True})
+        return newItemObj.get_plant_item_data()
 
     @classmethod
     def GetItemSummary(cls,item_type,item):
