@@ -21,7 +21,11 @@ class ReserveSummary(SalesInvBase):
         self.updating = kwargs.get('updating',False)
         self._item_reserves_tot = {}  # {"Plants":{"Belita":5,"Bonita":8},"Vase":{"CoolX":3}}
         self._item_reserves = {}  # {"Plants":{"Belita":5,"Bonita":8},"Vase":{"CoolX":3}}
-        base = "[*].{id: id, reserve_date: reserve_date, finish_week: finish_week, customer: customer.name, customer_id: customer.id, location: location.name, location_id: location.id, inventory_location: inventory_location, num_reserved: num_reserved, type: item.type item_name: item.name, item_id: item.id "
+        base1 = "[*].{id: id, reserve_date: reserve_date, finish_week: finish_week, customer: customer.name, "
+        base2 = "customer_id: customer.id, location: location.name, location_id: location.id, inventory_location: "
+        base3 = "inventory_location, num_reserved: num_reserved, type: item.type item_name: item.name, item_id: item.id"
+        base4 = ", custom_plant_item: custom_plant_item "
+        base = base1 + base2 + base3 + base4
         self.inventory_tracking = InventoryTracking.getInstance()
         self.update_jpath = base+","+self.inventory_tracking.jPath_Suffix+"}"
         
@@ -100,6 +104,7 @@ class ReserveSummary(SalesInvBase):
         
     def _refresh_all_reserves_inner(self,holdArr, docDict,docRef):
         docDict['id'] = docRef.id
+        docDict['custom_plant_item'] = docDict.get('custom_plant_item',False)
         holdArr.append(docDict)
 
     def _get_item_type_summary(self):
@@ -151,6 +156,10 @@ class ReserveSummary(SalesInvBase):
         entry['recipes'] = reserveDict.get('recipe_items',[])
         entry['num_reserved'] = reserveDict['num_reserved']
         entry['inventory_location'] = reserveDict.get('inventory_location',InventoryLocation.getInstance().default_location)
+        custItem = reserveDict.get('custom_plant_item',False)
+        if custItem is None or custItem == 'null' or custItem == '':
+            custItem = False
+        entry['custom_plant_item'] = custItem
         return entry
 
     def create_reserve_entry_ir(self,itemReserve):
@@ -162,6 +171,7 @@ class ReserveSummary(SalesInvBase):
             i['reserve_id'] = itemReserve['reserve_id']
             i['num_reserved'] = self._toNum(itemReserve.get('num_reserved',0),defaultNum=0)
             i['inventory_location'] = itemReserve['inventory_location']
+            i['custom_plant_item'] = itemReserve['custom_plant_item']
             qty = self._toNum(i.get('qty',0),defaultNum=0)
             i['total_reserved'] = qty * i['num_reserved']
         return itemArr
@@ -198,7 +208,9 @@ class ReserveSummary(SalesInvBase):
 
 
     def _apply_summary(self,item_reserve):
-        return jmespath.search(self.update_jpath+" | [0]",[item_reserve.get_dict()])
+        data = item_reserve.get_dict()
+        data['custom_plant_item'] = data.get('custom_plant_item',False)
+        return jmespath.search(self.update_jpath+" | [0]",[data])
     
     def get_itemTypeSummary(self,item_type):
         fsDoc = FSDocument(self.get_firestore_client().document(self.parent_path+"/"+item_type.replace(" ","_")))
